import osmnx as ox
import networkx as nx
import pickle
import time, datetime
import os.path
import os
import pandas as pd

from columnar import columnar

ox.settings.log_console=True
ox.settings.use_cache=True

print('OSMnx version: ', ox.__version__)
print('NetworkX version: ',nx.__version__)
print('Panda version: ',pd.__version__)

start_time = time.time()

def elapsed_time(start):
    end_time = time.time()
    duration = round(end_time - start)
    timestamp = str(datetime.timedelta(seconds = duration))
    print(timestamp)

def report(Q, txt=''):
    print(len(Q.nodes()), ' nodes ', txt)
    print(len(Q.edges()), ' edges ', txt)
    if isinstance(Q,nx.MultiGraph):    
        length = ox.stats.edge_length_total(Q) #in meters
        length = length * 0.0006213712
        print(f'total length {length:.2f} miles ', txt)
    zeroes = 0    
    odds = 0
    evens = 0
    twos = 0
    ones = 0
    for node in Q.nodes:
        d = Q.degree(node)
        if d == 0:
            zeroes += 1
        if d % 2 == 1:
            odds += 1
        else:
            evens += 1
        if d == 2:
            twos += 1
        if d == 1:
            ones += 1
    if zeroes > 0:            
        print('number of zero-degree nodes ' + txt + ' :', zeroes)            
    print('number of odd intersections ' + txt + ' :', odds)
    print('number of even intersections ' + txt + ' :', evens)
    print('number of 2-way intersections ' + txt + ' :', twos)
    print('number of trailhead/terminus ' + txt + ' :', ones)
    print('')    

#G = pickle.load(open('euler_graph.pkl', 'rb'))
G = pickle.load(open('max_candidate_with_towns_unsimplified.pkl', 'rb'))
if nx.is_eulerian(G):
    print('desimplified graph loaded from disk is still eulerian')
else:
    print('desimplified graph loaded from disk not eulerian-- debugging needed')

# saved from desimplify.py
gila_trail_plus_road_filename = 'gila_trail_plus_road_from_bbox.pkl'
GTR = pickle.load(open(gila_trail_plus_road_filename, 'rb'))


report(G,'euler graph unsimplified loaded from disk')



def report_path(Q,Q_orig, txt='',freedom_units=False,lat_long=False):
    meters_to_mile = 0.0006213712
    meters_to_feet = 3.28084
    meters_to_km = 0.001
    
    if nx.is_eulerian(Q):
        total_length = 0
        data = []
        
        for u,v,k in nx.eulerian_circuit(Q, keys=True): #fix: use predefined circuit, not arbitrary
             data_line = []
             if freedom_units:
                 data_line.append(str(round(total_length*meters_to_mile,1)))
             else:
                 data_line.append(str(round(total_length*meters_to_km,1)))
                 
             d = Q.get_edge_data(u,v)[k]
            # print(d)
             length=0                 
             if 'length' in d:
                 length = d['length'] #meters
                 total_length += length
             node_attr = Q.nodes(data=True)[v]
            # print( node_attr )
            
             street_count = node_attr['street_count']
             if street_count == 2:
                 continue
             
             long = node_attr['x']
             lat = node_attr['y']
             if lat_long:
                 data_line.append(lat)
                 data_line_append(long)
             #elevation = get_elevation(lat,long) #in meters
             if 'elevation' in node_attr:
                 elevation = node_attr['elevation']
                 if freedom_units:
                     data_line.append(str(round(elevation*meters_to_feet)))
                 else:
                     data_line.append(str(round(elevation)))
             else:
                 data_line.append(' ')

             trail_name=''
             if 'name' in d:
                 trail_name = d['name']
                 if not isinstance(trail_name, str):
                     trail_name = ','.join(trail_name) #in case is a list
             
             data_line.append(trail_name)                     
             
             crossing = set()
             crossing_txt = ''
             for x,neighbor,k,d in Q_orig.edges(v,data=True, keys=True):
                 if 'name' in d:
                     name = d['name']
                     if not isinstance(name, str): #FIX with a foreach
                         name = ','.join(name) #in case is a list
                     if name != trail_name:
                         crossing.add(name)
                 elif 'highway' in d:
                     name = d['highway']
                     name = '<' + name + '>'
                     crossing.add(name)
             if len(crossing)>0:
                 crossing_txt = ','.join(crossing)
             data_line.append(crossing_txt)
                              
             data.append(data_line)                 
        header = []
        if freedom_units:
            header.append('MILE')
        else:
            header.append('KM')
        if lat_long:
            header.append('LAT')
            header.append('LONG')
        header.append('ELEVATION')
        header.append('TRAIL')
        header.append('INTERSECTION')
        table = columnar(data, header, no_borders=True,terminal_width=132)
        print(table)     
    else:
        print('The graph is not eulerian')

# ---------------- add elevation to graph -------------------------
# in case graph is unsimplified, only add elevation to more important nodes
def add_elevation(Q):
    H = nx.MultiGraph()
    H.graph['crs'] = ox.settings.default_crs

    for node, dat in Q.nodes(data=True):
        if 'street_count' in dat:
            street_count = dat['street_count']
            if street_count > 2:
                H.add_node(node,**dat)
    ox.settings.elevation_url_template = \
        'https://api.open-elevation.com/api/v1/lookup?locations={locations}'
    ox.elevation.add_node_elevations_google(H, api_key=None,
        batch_size=350,
        pause=1.0,
            )
    print('Added elevation data for ',H.number_of_nodes(), ' nodes')
    for node,dat in H.nodes(data=True):
        Q.add_node(node,**dat)
    return Q

    


add_elevation(G)
pickle.dump(G,open('max_candidate_with_towns_unsimplified_with_elevation.pkl', 'wb'))

report_path(G,GTR,'Gila X Loop',freedom_units=True)
elapsed_time(start_time)
print('finished script ' + os.path.basename(__file__))
